/*
 * Performance-Monitoring Counters driver
 *
 * Copyright (C) 1999-2000  Mikael Pettersson
 */
#ifndef _LINUX_PERFCTR_H
#define _LINUX_PERFCTR_H

#include <asm/perfctr.h>

/* returned by the PERFCTR_INFO ioctl */
struct perfctr_info {
	unsigned char version_major;
	unsigned char version_minor;
	unsigned char version_micro;
	unsigned char nrcpus;
	unsigned char cpu_type;
	unsigned char cpu_features;
};
#define PERFCTR_CPU_INTEL_P5	0	/* no rdpmc */
#define PERFCTR_CPU_INTEL_P5MMX	1
#define PERFCTR_CPU_INTEL_P6	2
#define PERFCTR_CPU_INTEL_PII	3
#define PERFCTR_CPU_INTEL_PIII	4
#define PERFCTR_CPU_CYRIX_MII	5
#define PERFCTR_CPU_WINCHIP_C6	6	/* no rdtsc */
#define PERFCTR_CPU_WINCHIP_2	7	/* no rdtsc */
#define PERFCTR_CPU_AMD_K7	8

#define PERFCTR_FEATURE_RDPMC	0x01
#define PERFCTR_FEATURE_RDTSC	0x02

/* user-visible state of an mmap:ed virtual perfctr */
struct vperfctr_state {
	int status;	/* UNLINKED, STOPPED, or # of active counters */
	struct perfctr_control control;
	struct perfctr_sum_ctrs sum;
	struct perfctr_low_ctrs start;
};
#define VPERFCTR_STATUS_UNLINKED -1
#define VPERFCTR_STATUS_STOPPED	0

/* parameter in PERFCTR_GLOBAL_CONTROL ioctl */
struct gperfctr_control {
	unsigned long interval_usec;
	unsigned int nrcpus;
	struct perfctr_control cpu_control[0];
};

/* returned by PERFCTR_GLOBAL_READ ioctl */
struct gperfctr_cpu_state {
	int nrctrs;
	struct perfctr_control control;
	struct perfctr_sum_ctrs sum;
};
struct gperfctr_state {
	unsigned nrcpus;
	struct gperfctr_cpu_state cpu_state[0];
};

#ifdef __KERNEL__

#if defined(CONFIG_PERFCTR) || defined(CONFIG_PERFCTR_MODULE)

/* called from drivers/char/misc.c:misc_init() */
extern int perfctr_init(void);

#else	/* !CONFIG_PERFCTR */

static __inline__ int perfctr_init(void) { return 0; }

#endif	/* CONFIG_PERFCTR */

/*
 * Virtual per-process performance-monitoring counters.
 */
struct vperfctr;	/* opaque */

#if defined(CONFIG_PERFCTR_VIRTUAL)

/* process management callbacks */
extern void __vperfctr_unlink(struct vperfctr*);
extern void __vperfctr_suspend(struct vperfctr*);
extern void __vperfctr_resume(struct vperfctr*);

#ifdef CONFIG_PERFCTR_MODULE
extern struct vperfctr_stub {
	void (*unlink)(struct vperfctr*);
	void (*suspend)(struct vperfctr*);
	void (*resume)(struct vperfctr*);
} vperfctr_stub;
#define _vperfctr_unlink(x)	vperfctr_stub.unlink((x))
#define _vperfctr_suspend(x)	vperfctr_stub.suspend((x))
#define _vperfctr_resume(x)	vperfctr_stub.resume((x))
#else	/* !CONFIG_PERFCTR_MODULE */
#define _vperfctr_unlink(x)	__vperfctr_unlink((x))
#define _vperfctr_suspend(x)	__vperfctr_suspend((x))
#define _vperfctr_resume(x)	__vperfctr_resume((x))
#endif	/* CONFIG_PERFCTR_MODULE */

#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,11)
#define TASK_PERFCTR(tsk)	((tsk)->tss.perfctr)
#else
#define TASK_PERFCTR(tsk)	((tsk)->thread.perfctr)
#endif

static __inline__ void perfctr_init_task(struct task_struct *tsk)
{
	TASK_PERFCTR(tsk) = NULL;
}

static __inline__ void perfctr_exit_task(struct task_struct *tsk)
{
	struct vperfctr *perfctr = TASK_PERFCTR(tsk);
	if( perfctr ) {
		TASK_PERFCTR(tsk) = NULL;
		_vperfctr_unlink(perfctr);
	}
}

static __inline__ void perfctr_suspend_thread(struct thread_struct *prev)
{
	struct vperfctr *perfctr = prev->perfctr;
	if( perfctr )
		_vperfctr_suspend(perfctr);
}

/* PRE: next is current */
static __inline__ void perfctr_resume_thread(struct thread_struct *next)
{
	struct vperfctr *perfctr = next->perfctr;
	if( perfctr )
		_vperfctr_resume(perfctr);
}

#else	/* !CONFIG_PERFCTR_VIRTUAL */

static __inline__ void perfctr_init_task(struct task_struct *p) { }
static __inline__ void perfctr_exit_task(struct task_struct *p) { }
static __inline__ void perfctr_suspend_thread(struct thread_struct *t) { }
static __inline__ void perfctr_resume_thread(struct thread_struct *t) { }

#endif	/* CONFIG_PERFCTR_VIRTUAL */

#endif	/* __KERNEL__ */

#include <linux/ioctl.h>
#define _PERFCTR_IOCTL	0xD0	/* 'P'+128, currently unassigned */

#define PERFCTR_INFO		_IOR(_PERFCTR_IOCTL,0,struct perfctr_info)
#define PERFCTR_ATTACH_RDONLY	_IOW(_PERFCTR_IOCTL,1,int)	/* not ptr! */
#define PERFCTR_ATTACH_RDWR	_IOW(_PERFCTR_IOCTL,2,int)	/* not ptr! */
#define PERFCTR_VIRTUAL_SAMPLE	 _IO(_PERFCTR_IOCTL,3)
#define PERFCTR_VIRTUAL_UNLINK	 _IO(_PERFCTR_IOCTL,4)
#define PERFCTR_VIRTUAL_CONTROL	_IOW(_PERFCTR_IOCTL,5,struct perfctr_control)
#define PERFCTR_VIRTUAL_STOP	 _IO(_PERFCTR_IOCTL,6)
#define PERFCTR_GLOBAL_CONTROL	_IOW(_PERFCTR_IOCTL,7,struct gperfctr_control)
#define PERFCTR_GLOBAL_READ	_IOR(_PERFCTR_IOCTL,8,struct gperfctr_state)
#define PERFCTR_GLOBAL_STOP	 _IO(_PERFCTR_IOCTL,9)

#endif	/* _LINUX_PERFCTR_H */
